/*!
* rete-history-plugin v2.0.2
* (c) 2025 Vitaliy Stoliarov
* Released under the MIT license.
* */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _asyncToGenerator = require('@babel/runtime/helpers/asyncToGenerator');
var _toConsumableArray = require('@babel/runtime/helpers/toConsumableArray');
var _classCallCheck = require('@babel/runtime/helpers/classCallCheck');
var _createClass = require('@babel/runtime/helpers/createClass');
var _possibleConstructorReturn = require('@babel/runtime/helpers/possibleConstructorReturn');
var _getPrototypeOf = require('@babel/runtime/helpers/getPrototypeOf');
var _get = require('@babel/runtime/helpers/get');
var _inherits = require('@babel/runtime/helpers/inherits');
var _defineProperty = require('@babel/runtime/helpers/defineProperty');
var _regeneratorRuntime = require('@babel/runtime/regenerator');
var rete = require('rete');
var reteAreaPlugin = require('rete-area-plugin');
var _typeof = require('@babel/runtime/helpers/typeof');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var _asyncToGenerator__default = /*#__PURE__*/_interopDefaultLegacy(_asyncToGenerator);
var _toConsumableArray__default = /*#__PURE__*/_interopDefaultLegacy(_toConsumableArray);
var _classCallCheck__default = /*#__PURE__*/_interopDefaultLegacy(_classCallCheck);
var _createClass__default = /*#__PURE__*/_interopDefaultLegacy(_createClass);
var _possibleConstructorReturn__default = /*#__PURE__*/_interopDefaultLegacy(_possibleConstructorReturn);
var _getPrototypeOf__default = /*#__PURE__*/_interopDefaultLegacy(_getPrototypeOf);
var _get__default = /*#__PURE__*/_interopDefaultLegacy(_get);
var _inherits__default = /*#__PURE__*/_interopDefaultLegacy(_inherits);
var _defineProperty__default = /*#__PURE__*/_interopDefaultLegacy(_defineProperty);
var _regeneratorRuntime__default = /*#__PURE__*/_interopDefaultLegacy(_regeneratorRuntime);
var _typeof__default = /*#__PURE__*/_interopDefaultLegacy(_typeof);

var History = /*#__PURE__*/function () {
  function History(_ref) {
    var limit = _ref.limit;
    _classCallCheck__default["default"](this, History);
    _defineProperty__default["default"](this, "active", false);
    _defineProperty__default["default"](this, "produced", []);
    _defineProperty__default["default"](this, "reserved", []);
    if (limit && typeof limit === 'number') this.limit = limit;
  }
  return _createClass__default["default"](History, [{
    key: "add",
    value: function add(action) {
      if (this.active) return;
      this.produced.push({
        time: Date.now(),
        action: action
      });
      if (this.limit && this.produced.length > this.limit) this.produced.shift();
      this.reserved = [];
    }
  }, {
    key: "getRecent",
    value: function getRecent(ms) {
      var now = Date.now();
      var treshold = now - ms;
      var list = [];
      for (var i = this.produced.length - 1; i >= 0; i--) {
        var action = this.produced[i];
        if (action.time <= treshold) break;
        if (action.separated) break;
        list.push(action);
      }
      return list;
    }
  }, {
    key: "move",
    value: function () {
      var _move = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee(from, to, type) {
        var record;
        return _regeneratorRuntime__default["default"].wrap(function _callee$(_context) {
          while (1) switch (_context.prev = _context.next) {
            case 0:
              record = from.pop();
              if (record) {
                _context.next = 3;
                break;
              }
              return _context.abrupt("return");
            case 3:
              this.active = true;
              _context.next = 6;
              return record.action[type]();
            case 6:
              to.push(record);
              this.active = false;
              return _context.abrupt("return", record);
            case 9:
            case "end":
              return _context.stop();
          }
        }, _callee, this);
      }));
      function move(_x, _x2, _x3) {
        return _move.apply(this, arguments);
      }
      return move;
    }()
  }, {
    key: "undo",
    value: function () {
      var _undo = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee2() {
        return _regeneratorRuntime__default["default"].wrap(function _callee2$(_context2) {
          while (1) switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return this.move(this.produced, this.reserved, 'undo');
            case 2:
              return _context2.abrupt("return", _context2.sent);
            case 3:
            case "end":
              return _context2.stop();
          }
        }, _callee2, this);
      }));
      function undo() {
        return _undo.apply(this, arguments);
      }
      return undo;
    }()
  }, {
    key: "clear",
    value: function clear() {
      this.active = false;
      this.produced = [];
      this.reserved = [];
    }
  }, {
    key: "redo",
    value: function () {
      var _redo = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee3() {
        return _regeneratorRuntime__default["default"].wrap(function _callee3$(_context3) {
          while (1) switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return this.move(this.reserved, this.produced, 'redo');
            case 2:
              return _context3.abrupt("return", _context3.sent);
            case 3:
            case "end":
              return _context3.stop();
          }
        }, _callee3, this);
      }));
      function redo() {
        return _redo.apply(this, arguments);
      }
      return redo;
    }()
  }]);
}();

/**
 * Adds keyboard shortcuts for history undo/redo
 * @param plugin History plugin instance
 */
function keyboard(plugin) {
  document.addEventListener('keydown', function (e) {
    if (!e.ctrlKey && !e.metaKey) return;
    switch (e.code) {
      case 'KeyZ':
        void plugin.undo();
        break;
      case 'KeyY':
        void plugin.redo();
        break;
    }
  });
}

/**
 * History plugin extensions
 * @module
 */

var index$2 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  keyboard: keyboard
});

var AddConnectionAction = /*#__PURE__*/function () {
  function AddConnectionAction(editor, connection) {
    _classCallCheck__default["default"](this, AddConnectionAction);
    this.editor = editor;
    this.connection = connection;
  }
  return _createClass__default["default"](AddConnectionAction, [{
    key: "undo",
    value: function () {
      var _undo = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee() {
        return _regeneratorRuntime__default["default"].wrap(function _callee$(_context) {
          while (1) switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return this.editor.removeConnection(this.connection.id);
            case 2:
            case "end":
              return _context.stop();
          }
        }, _callee, this);
      }));
      function undo() {
        return _undo.apply(this, arguments);
      }
      return undo;
    }()
  }, {
    key: "redo",
    value: function () {
      var _redo = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee2() {
        return _regeneratorRuntime__default["default"].wrap(function _callee2$(_context2) {
          while (1) switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return this.editor.addConnection(this.connection);
            case 2:
            case "end":
              return _context2.stop();
          }
        }, _callee2, this);
      }));
      function redo() {
        return _redo.apply(this, arguments);
      }
      return redo;
    }()
  }]);
}();
var RemoveConnectionAction = /*#__PURE__*/function () {
  function RemoveConnectionAction(editor, connection) {
    _classCallCheck__default["default"](this, RemoveConnectionAction);
    this.editor = editor;
    this.connection = connection;
  }
  return _createClass__default["default"](RemoveConnectionAction, [{
    key: "undo",
    value: function () {
      var _undo2 = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee3() {
        return _regeneratorRuntime__default["default"].wrap(function _callee3$(_context3) {
          while (1) switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return this.editor.addConnection(this.connection);
            case 2:
            case "end":
              return _context3.stop();
          }
        }, _callee3, this);
      }));
      function undo() {
        return _undo2.apply(this, arguments);
      }
      return undo;
    }()
  }, {
    key: "redo",
    value: function () {
      var _redo2 = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee4() {
        return _regeneratorRuntime__default["default"].wrap(function _callee4$(_context4) {
          while (1) switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return this.editor.removeConnection(this.connection.id);
            case 2:
            case "end":
              return _context4.stop();
          }
        }, _callee4, this);
      }));
      function redo() {
        return _redo2.apply(this, arguments);
      }
      return redo;
    }()
  }]);
}();

function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty__default["default"](e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
var AddNodeAction = /*#__PURE__*/function () {
  function AddNodeAction(editor, area, nodeId) {
    _classCallCheck__default["default"](this, AddNodeAction);
    this.editor = editor;
    this.area = area;
    this.nodeId = nodeId;
  }
  return _createClass__default["default"](AddNodeAction, [{
    key: "undo",
    value: function () {
      var _undo = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee() {
        var _this$area$nodeViews$;
        return _regeneratorRuntime__default["default"].wrap(function _callee$(_context) {
          while (1) switch (_context.prev = _context.next) {
            case 0:
              this.node = this.editor.getNode(this.nodeId);
              this.position = (_this$area$nodeViews$ = this.area.nodeViews.get(this.nodeId)) === null || _this$area$nodeViews$ === void 0 ? void 0 : _this$area$nodeViews$.position;
              _context.next = 4;
              return this.editor.removeNode(this.nodeId);
            case 4:
            case "end":
              return _context.stop();
          }
        }, _callee, this);
      }));
      function undo() {
        return _undo.apply(this, arguments);
      }
      return undo;
    }()
  }, {
    key: "redo",
    value: function () {
      var _redo = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee2() {
        return _regeneratorRuntime__default["default"].wrap(function _callee2$(_context2) {
          while (1) switch (_context2.prev = _context2.next) {
            case 0:
              if (!this.node) {
                _context2.next = 3;
                break;
              }
              _context2.next = 3;
              return this.editor.addNode(this.node);
            case 3:
              if (!(this.node && this.position)) {
                _context2.next = 6;
                break;
              }
              _context2.next = 6;
              return this.area.translate(this.node.id, this.position);
            case 6:
            case "end":
              return _context2.stop();
          }
        }, _callee2, this);
      }));
      function redo() {
        return _redo.apply(this, arguments);
      }
      return redo;
    }()
  }]);
}();
var RemoveNodeAction = /*#__PURE__*/function () {
  function RemoveNodeAction(editor, area, node, position) {
    _classCallCheck__default["default"](this, RemoveNodeAction);
    this.editor = editor;
    this.area = area;
    this.node = node;
    this.position = position;
  }
  return _createClass__default["default"](RemoveNodeAction, [{
    key: "undo",
    value: function () {
      var _undo2 = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee3() {
        return _regeneratorRuntime__default["default"].wrap(function _callee3$(_context3) {
          while (1) switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return this.editor.addNode(this.node);
            case 2:
              _context3.next = 4;
              return this.area.translate(this.node.id, this.position);
            case 4:
            case "end":
              return _context3.stop();
          }
        }, _callee3, this);
      }));
      function undo() {
        return _undo2.apply(this, arguments);
      }
      return undo;
    }()
  }, {
    key: "redo",
    value: function () {
      var _redo2 = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee4() {
        return _regeneratorRuntime__default["default"].wrap(function _callee4$(_context4) {
          while (1) switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return this.editor.removeNode(this.node.id);
            case 2:
            case "end":
              return _context4.stop();
          }
        }, _callee4, this);
      }));
      function redo() {
        return _redo2.apply(this, arguments);
      }
      return redo;
    }()
  }]);
}();
var DragNodeAction = /*#__PURE__*/function () {
  function DragNodeAction(area, nodeId, prev) {
    _classCallCheck__default["default"](this, DragNodeAction);
    this.area = area;
    this.nodeId = nodeId;
    var view = area.nodeViews.get(nodeId);
    if (!view) return;
    this.prev = _objectSpread({}, prev);
    this["new"] = _objectSpread({}, view.position);
  }
  return _createClass__default["default"](DragNodeAction, [{
    key: "translate",
    value: function () {
      var _translate = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee5(position) {
        var view;
        return _regeneratorRuntime__default["default"].wrap(function _callee5$(_context5) {
          while (1) switch (_context5.prev = _context5.next) {
            case 0:
              view = this.area.nodeViews.get(this.nodeId);
              if (view) {
                _context5.next = 3;
                break;
              }
              return _context5.abrupt("return");
            case 3:
              _context5.next = 5;
              return view.translate(position.x, position.y);
            case 5:
            case "end":
              return _context5.stop();
          }
        }, _callee5, this);
      }));
      function translate(_x) {
        return _translate.apply(this, arguments);
      }
      return translate;
    }()
  }, {
    key: "undo",
    value: function () {
      var _undo3 = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee6() {
        return _regeneratorRuntime__default["default"].wrap(function _callee6$(_context6) {
          while (1) switch (_context6.prev = _context6.next) {
            case 0:
              _context6.next = 2;
              return this.translate(this.prev);
            case 2:
            case "end":
              return _context6.stop();
          }
        }, _callee6, this);
      }));
      function undo() {
        return _undo3.apply(this, arguments);
      }
      return undo;
    }()
  }, {
    key: "redo",
    value: function () {
      var _redo3 = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee7() {
        return _regeneratorRuntime__default["default"].wrap(function _callee7$(_context7) {
          while (1) switch (_context7.prev = _context7.next) {
            case 0:
              _context7.next = 2;
              return this.translate(this["new"]);
            case 2:
            case "end":
              return _context7.stop();
          }
        }, _callee7, this);
      }));
      function redo() {
        return _redo3.apply(this, arguments);
      }
      return redo;
    }()
  }]);
}();

function trackNodes(history, props) {
  var nodes = new Map();
  var positions = new Map();
  var area = history.parentScope(reteAreaPlugin.BaseAreaPlugin);
  var editor = area.parentScope(rete.NodeEditor);
  var timing = props.timing;

  // eslint-disable-next-line max-statements
  editor.addPipe(function (context) {
    if (context.type === 'nodecreated') {
      var id = context.data.id;
      history.add(new AddNodeAction(editor, area, id));
      nodes.set(id, editor.getNode(context.data.id));
    }
    if (context.type === 'noderemoved') {
      var _id = context.data.id;
      var node = nodes.get(_id);
      var position = positions.get(_id);
      if (!node) throw new Error('node');
      if (!position) throw new Error('position' + _id);
      history.add(new RemoveNodeAction(editor, area, node, position));
      positions["delete"](_id);
      nodes["delete"](_id);
    }
    return context;
  });
  area.addPipe(function (context) {
    if (!('type' in context)) return context;
    if (context.type === 'nodetranslated') {
      var _context$data = context.data,
        id = _context$data.id,
        position = _context$data.position;
      positions.set(id, position);
    }
    return context;
  });
  var picked = [];

  // eslint-disable-next-line max-statements
  area.addPipe(function (context) {
    if (!context || _typeof__default["default"](context) !== 'object' || !('type' in context)) return context;
    if (context.type === 'nodepicked') {
      picked.push(context.data.id);
    }
    if (context.type === 'nodedragged') {
      var index = picked.indexOf(context.data.id);
      if (index >= 0) picked.splice(index, 1);
    }
    if (context.type === 'nodetranslated') {
      var _context$data2 = context.data,
        id = _context$data2.id,
        position = _context$data2.position,
        previous = _context$data2.previous;
      var recent = history.getRecent(timing).filter(function (n) {
        return n.action instanceof DragNodeAction;
      }).filter(function (n) {
        return n.action.nodeId === id;
      });
      if (recent.length > 1) throw new Error('> 1');
      if (recent[0]) {
        recent[0].action["new"] = position;
        recent[0].time = Date.now();
      } else {
        history.add(new DragNodeAction(area, id, previous));
      }
    }
    return context;
  });
}
function trackConnections(history) {
  var connections = new Map();
  var editor = history.parentScope().parentScope(rete.NodeEditor);
  editor.addPipe(function (context) {
    if (context.type === 'connectioncreated') {
      var connection = editor.getConnection(context.data.id);
      history.add(new AddConnectionAction(editor, connection));
      connections.set(context.data.id, connection);
    }
    if (context.type === 'connectionremoved') {
      var _connection = connections.get(context.data.id);
      if (_connection) {
        history.add(new RemoveConnectionAction(editor, _connection));
      }
    }
    return context;
  });
}

/**
 * Classic preset for the history plugin. Tracks node adding/removing/translating, connection adding/removing.
 */
function setup(props) {
  return {
    connect: function connect(history) {
      var _props$timing;
      var timing = (_props$timing = props === null || props === void 0 ? void 0 : props.timing) !== null && _props$timing !== void 0 ? _props$timing : history.timing * 2;
      trackNodes(history, {
        timing: timing
      });
      trackConnections(history);
    }
  };
}

var index$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  AddConnectionAction: AddConnectionAction,
  AddNodeAction: AddNodeAction,
  DragNodeAction: DragNodeAction,
  RemoveConnectionAction: RemoveConnectionAction,
  RemoveNodeAction: RemoveNodeAction,
  setup: setup
});

/**
 * Presets for the history plugin
 * @module
 */

var index = /*#__PURE__*/Object.freeze({
  __proto__: null,
  classic: index$1
});

function _callSuper(t, o, e) { return o = _getPrototypeOf__default["default"](o), _possibleConstructorReturn__default["default"](t, _isNativeReflectConstruct() ? Reflect.construct(o, e || [], _getPrototypeOf__default["default"](t).constructor) : o.apply(t, e)); }
function _isNativeReflectConstruct() { try { var t = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); } catch (t) {} return (_isNativeReflectConstruct = function _isNativeReflectConstruct() { return !!t; })(); }
function _superPropGet(t, e, o, r) { var p = _get__default["default"](_getPrototypeOf__default["default"](1 & r ? t.prototype : t), e, o); return 2 & r && "function" == typeof p ? function (t) { return p.apply(o, t); } : p; }
/**
 * History plugin. Allows to undo/redo actions such as adding/removing nodes, connections, etc.
 */
var HistoryPlugin = /*#__PURE__*/function (_Scope) {
  /**
   * @constructor
   * @param props History plugin properties
   * @param props.timing Time in ms between actions to be considered as one action
   */
  function HistoryPlugin(props) {
    var _props$timing;
    var _this;
    _classCallCheck__default["default"](this, HistoryPlugin);
    _this = _callSuper(this, HistoryPlugin, ['history']);
    _defineProperty__default["default"](_this, "history", new History({}));
    _defineProperty__default["default"](_this, "presets", []);
    _this.timing = (_props$timing = props === null || props === void 0 ? void 0 : props.timing) !== null && _props$timing !== void 0 ? _props$timing : 200;
    return _this;
  }
  _inherits__default["default"](HistoryPlugin, _Scope);
  return _createClass__default["default"](HistoryPlugin, [{
    key: "setParent",
    value: function setParent(scope) {
      var _this2 = this;
      _superPropGet(HistoryPlugin, "setParent", this, 3)([scope]);
      this.area = this.parentScope(reteAreaPlugin.BaseAreaPlugin);
      this.editor = this.area.parentScope(rete.NodeEditor);
      this.presets.forEach(function (preset) {
        preset.connect(_this2);
      });
      this.editor.addPipe(function (context) {
        if (context.type === 'cleared') {
          _this2.history.clear();
        }
        return context;
      });
    }

    /**
     * Adds a preset to the history plugin
     * @param preset Preset that manages some actions, e.g. classic preset for adding/removing nodes, connections, etc.
     */
  }, {
    key: "addPreset",
    value: function addPreset(preset) {
      this.presets.push(preset);
      if (this.area && this.editor) preset.connect(this);
    }

    /**
     * Adds an action to the history
     * @param action Action instance
     */
  }, {
    key: "add",
    value: function add(action) {
      this.history.add(action);
    }

    /**
     * Get produced history records
     * @returns History snapshot
     */
  }, {
    key: "getHistorySnapshot",
    value: function getHistorySnapshot() {
      return _toConsumableArray__default["default"](this.history.produced);
    }

    /**
     * Get recent history records produced in the last `ms` milliseconds
     * @param ms Delta time in milliseconds
     * @returns History records produced in the last `ms` milliseconds
     */
  }, {
    key: "getRecent",
    value: function getRecent(ms) {
      return this.history.getRecent(ms);
    }

    /**
     * Clear history
     */
  }, {
    key: "clear",
    value: function clear() {
      this.history.clear();
    }

    /**
     *
     */
  }, {
    key: "separate",
    value: function separate() {
      var latest = this.history.produced[this.history.produced.length - 1];
      if (latest) latest.separated = true;
    }

    /**
     * Asynchronously undoes an actions until the `separated` flag is encountered or the time between the current and previous action exceeds `timing` property which is 200ms by default
     */
  }, {
    key: "undo",
    value: (function () {
      var _undo = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee() {
        var record, latest;
        return _regeneratorRuntime__default["default"].wrap(function _callee$(_context) {
          while (1) switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return this.history.undo();
            case 2:
              record = _context.sent;
              if (!record) {
                _context.next = 8;
                break;
              }
              latest = this.history.produced[this.history.produced.length - 1];
              if (!(latest && !latest.separated && latest.time + this.timing > record.time)) {
                _context.next = 8;
                break;
              }
              _context.next = 8;
              return this.undo();
            case 8:
            case "end":
              return _context.stop();
          }
        }, _callee, this);
      }));
      function undo() {
        return _undo.apply(this, arguments);
      }
      return undo;
    }()
    /**
     * Asynchronously redoes an actions until the `separated` flag is encountered or the time between the current and previous action exceeds `timing` property which is 200ms by default
     */
    )
  }, {
    key: "redo",
    value: (function () {
      var _redo = _asyncToGenerator__default["default"](/*#__PURE__*/_regeneratorRuntime__default["default"].mark(function _callee2() {
        var record, latest;
        return _regeneratorRuntime__default["default"].wrap(function _callee2$(_context2) {
          while (1) switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return this.history.redo();
            case 2:
              record = _context2.sent;
              if (!record) {
                _context2.next = 8;
                break;
              }
              latest = this.history.reserved[this.history.reserved.length - 1];
              if (!(latest && !record.separated && record.time + this.timing > latest.time)) {
                _context2.next = 8;
                break;
              }
              _context2.next = 8;
              return this.redo();
            case 8:
            case "end":
              return _context2.stop();
          }
        }, _callee2, this);
      }));
      function redo() {
        return _redo.apply(this, arguments);
      }
      return redo;
    }())
  }]);
}(rete.Scope);

exports.HistoryExtensions = index$2;
exports.HistoryPlugin = HistoryPlugin;
exports.Presets = index;
//# sourceMappingURL=rete-history-plugin.common.js.map
